@@ -39,18 +39,14 @@ from .._exceptions import BadTypeInNameException
 from .._logger import log
 from .._protocol.outgoing import DNSOutgoing
 from .._updates import RecordUpdate, RecordUpdateListener
-from .._utils.asyncio import (
-    get_running_loop,
-    run_coro_with_timeout,
-    wait_event_or_timeout,
-)
+from .._utils.asyncio import get_running_loop, run_coro_with_timeout
 from .._utils.name import service_type_name
 from .._utils.net import IPVersion, _encode_address
 from .._utils.time import current_time_millis, millis_to_seconds
 from ..const import (
     _ADDRESS_RECORD_TYPES,
     _CLASS_IN,
-    _CLASS_UNIQUE,
+    _CLASS_IN_UNIQUE,
     _DNS_HOST_TTL,
     _DNS_OTHER_TTL,
     _FLAGS_QR_QUERY,
@@ -80,11 +76,11 @@ if TYPE_CHECKING:
     from .._core import Zeroconf
 
 
-def instance_name_from_service_info(info: "ServiceInfo") -> str:
+def instance_name_from_service_info(info: "ServiceInfo", strict: bool = True) -> str:
     """Calculate the instance name from the ServiceInfo."""
     # This is kind of funky because of the subtype based tests
     # need to make subtypes a first class citizen
-    service_name = service_type_name(info.name)
+    service_name = service_type_name(info.name, strict=strict)
     if not info.type.endswith(service_name):
         raise BadTypeInNameException
     return info.name[: -len(service_name) - 1]
@@ -93,6 +89,12 @@ def instance_name_from_service_info(info: "ServiceInfo") -> str:
 _cached_ip_addresses = lru_cache(maxsize=256)(ip_address)
 
 
+def _set_future_none_if_not_done(fut: asyncio.Future) -> None:
+    """Set a future to None if it is not done."""
+    if not fut.done():  # pragma: no branch
+        fut.set_result(None)
+
+
 class ServiceInfo(RecordUpdateListener):
     """Service information.
 
@@ -131,6 +133,7 @@ class ServiceInfo(RecordUpdateListener):
         "host_ttl",
         "other_ttl",
         "interface_index",
+        "_new_records_futures",
     )
 
     def __init__(
@@ -169,7 +172,7 @@ class ServiceInfo(RecordUpdateListener):
         self.priority = priority
         self.server = server if server else None
         self.server_key = server.lower() if server else None
-        self._properties: Dict[Union[str, bytes], Optional[Union[str, bytes]]] = {}
+        self._properties: Optional[Dict[Union[str, bytes], Optional[Union[str, bytes]]]] = None
         if isinstance(properties, bytes):
             self._set_text(properties)
         else:
@@ -177,7 +180,7 @@ class ServiceInfo(RecordUpdateListener):
         self.host_ttl = host_ttl
         self.other_ttl = other_ttl
         self.interface_index = interface_index
-        self._notify_event: Optional[asyncio.Event] = None
+        self._new_records_futures: List[asyncio.Future] = []
 
     @property
     def name(self) -> str:
@@ -223,7 +226,7 @@ class ServiceInfo(RecordUpdateListener):
                 self._ipv6_addresses.append(addr)
 
     @property
-    def properties(self) -> Dict:
+    def properties(self) -> Dict[Union[str, bytes], Optional[Union[str, bytes]]]:
         """If properties were set in the constructor this property returns the original dictionary
         of type `Dict[Union[bytes, str], Any]`.
 
@@ -231,13 +234,22 @@ class ServiceInfo(RecordUpdateListener):
         bytes and the values are either bytes, if there was a value, even empty, or `None`, if there
         was none. No further decoding is attempted. The type returned is `Dict[bytes, Optional[bytes]]`.
         """
+        if self._properties is None:
+            self._unpack_text_into_properties()
+        if TYPE_CHECKING:
+            assert self._properties is not None
         return self._properties
 
     async def async_wait(self, timeout: float) -> None:
         """Calling task waits for a given number of milliseconds or until notified."""
-        if self._notify_event is None:
-            self._notify_event = asyncio.Event()
-        await wait_event_or_timeout(self._notify_event, timeout=millis_to_seconds(timeout))
+        loop = asyncio.get_running_loop()
+        future = loop.create_future()
+        self._new_records_futures.append(future)
+        handle = loop.call_later(millis_to_seconds(timeout), _set_future_none_if_not_done, future)
+        try:
+            await future
+        finally:
+            handle.cancel()
 
     def addresses_by_version(self, version: IPVersion) -> List[bytes]:
         """List addresses matching IP version.
@@ -309,10 +321,10 @@ class ServiceInfo(RecordUpdateListener):
             for addr in self._ip_addresses_by_version_value(version.value)
         ]
 
-    def _set_properties(self, properties: Dict) -> None:
+    def _set_properties(self, properties: Dict[Union[str, bytes], Optional[Union[str, bytes]]]) -> None:
         """Sets properties and text of this info from a dictionary"""
         self._properties = properties
-        list_ = []
+        list_: List[bytes] = []
         result = b''
         for key, value in properties.items():
             if isinstance(key, str):
@@ -330,14 +342,25 @@ class ServiceInfo(RecordUpdateListener):
 
     def _set_text(self, text: bytes) -> None:
         """Sets properties and text given a text field"""
+        if text == self.text:
+            return
         self.text = text
+        # Clear the properties cache
+        self._properties = None
+
+    def _unpack_text_into_properties(self) -> None:
+        """Unpacks the text field into properties"""
+        text = self.text
         end = len(text)
         if end == 0:
+            # Properties should be set atomically
+            # in case another thread is reading them
             self._properties = {}
             return
+
         result: Dict[Union[str, bytes], Optional[Union[str, bytes]]] = {}
         index = 0
-        strs = []
+        strs: List[bytes] = []
         while index < end:
             length = text[index]
             index += 1
@@ -347,22 +370,25 @@ class ServiceInfo(RecordUpdateListener):
         key: bytes
         value: Optional[bytes]
         for s in strs:
-            try:
-                key, value = s.split(b'=', 1)
-            except ValueError:
+            key_value = s.split(b'=', 1)
+            if len(key_value) == 2:
+                key, value = key_value
+            else:
                 # No equals sign at all
                 key = s
                 value = None
 
             # Only update non-existent properties
-            if key and result.get(key) is None:
+            if key and key not in result:
                 result[key] = value
 
+        # Properties should be set atomically
+        # in case another thread is reading them
         self._properties = result
 
     def get_name(self) -> str:
         """Name accessor"""
-        return self.name[: len(self.name) - len(self.type) - 1]
+        return self._name[: len(self._name) - len(self.type) - 1]
 
     def _get_ip_addresses_from_cache_lifo(
         self, zc: 'Zeroconf', now: float, type: int
@@ -383,15 +409,21 @@ class ServiceInfo(RecordUpdateListener):
 
     def _set_ipv6_addresses_from_cache(self, zc: 'Zeroconf', now: float) -> None:
         """Set IPv6 addresses from the cache."""
-        self._ipv6_addresses = cast(
-            "List[IPv6Address]", self._get_ip_addresses_from_cache_lifo(zc, now, _TYPE_AAAA)
-        )
+        if TYPE_CHECKING:
+            self._ipv6_addresses = cast(
+                "List[IPv6Address]", self._get_ip_addresses_from_cache_lifo(zc, now, _TYPE_AAAA)
+            )
+        else:
+            self._ipv6_addresses = self._get_ip_addresses_from_cache_lifo(zc, now, _TYPE_AAAA)
 
     def _set_ipv4_addresses_from_cache(self, zc: 'Zeroconf', now: float) -> None:
         """Set IPv4 addresses from the cache."""
-        self._ipv4_addresses = cast(
-            "List[IPv4Address]", self._get_ip_addresses_from_cache_lifo(zc, now, _TYPE_A)
-        )
+        if TYPE_CHECKING:
+            self._ipv4_addresses = cast(
+                "List[IPv4Address]", self._get_ip_addresses_from_cache_lifo(zc, now, _TYPE_A)
+            )
+        else:
+            self._ipv4_addresses = self._get_ip_addresses_from_cache_lifo(zc, now, _TYPE_A)
 
     def update_record(self, zc: 'Zeroconf', now: float, record: Optional[DNSRecord]) -> None:
         """Updates service information from a DNS record.
@@ -409,9 +441,11 @@ class ServiceInfo(RecordUpdateListener):
 
         This method will be run in the event loop.
         """
-        if self._process_records_threadsafe(zc, now, records) and self._notify_event:
-            self._notify_event.set()
-            self._notify_event.clear()
+        if self._process_records_threadsafe(zc, now, records) and self._new_records_futures:
+            for future in self._new_records_futures:
+                if not future.done():
+                    future.set_result(None)
+            self._new_records_futures.clear()
 
     def _process_records_threadsafe(self, zc: 'Zeroconf', now: float, records: List[RecordUpdate]) -> bool:
         """Thread safe record updating.
@@ -495,9 +529,9 @@ class ServiceInfo(RecordUpdateListener):
         created: Optional[float] = None,
     ) -> List[DNSAddress]:
         """Return matching DNSAddress from ServiceInfo."""
-        name = self.server or self.name
+        name = self.server or self._name
         ttl = override_ttl if override_ttl is not None else self.host_ttl
-        class_ = _CLASS_IN | _CLASS_UNIQUE
+        class_ = _CLASS_IN_UNIQUE
         version_value = version.value
         return [
             DNSAddress(
@@ -518,30 +552,33 @@ class ServiceInfo(RecordUpdateListener):
             _TYPE_PTR,
             _CLASS_IN,
             override_ttl if override_ttl is not None else self.other_ttl,
-            self.name,
+            self._name,
             created,
         )
 
     def dns_service(self, override_ttl: Optional[int] = None, created: Optional[float] = None) -> DNSService:
         """Return DNSService from ServiceInfo."""
+        port = self.port
+        if TYPE_CHECKING:
+            assert isinstance(port, int)
         return DNSService(
-            self.name,
+            self._name,
             _TYPE_SRV,
-            _CLASS_IN | _CLASS_UNIQUE,
+            _CLASS_IN_UNIQUE,
             override_ttl if override_ttl is not None else self.host_ttl,
             self.priority,
             self.weight,
-            cast(int, self.port),
-            self.server or self.name,
+            port,
+            self.server or self._name,
             created,
         )
 
     def dns_text(self, override_ttl: Optional[int] = None, created: Optional[float] = None) -> DNSText:
         """Return DNSText from ServiceInfo."""
         return DNSText(
-            self.name,
+            self._name,
             _TYPE_TXT,
-            _CLASS_IN | _CLASS_UNIQUE,
+            _CLASS_IN_UNIQUE,
             override_ttl if override_ttl is not None else self.other_ttl,
             self.text,
             created,
@@ -552,11 +589,11 @@ class ServiceInfo(RecordUpdateListener):
     ) -> DNSNsec:
         """Return DNSNsec from ServiceInfo."""
         return DNSNsec(
-            self.name,
+            self._name,
             _TYPE_NSEC,
-            _CLASS_IN | _CLASS_UNIQUE,
+            _CLASS_IN_UNIQUE,
             override_ttl if override_ttl is not None else self.host_ttl,
-            self.name,
+            self._name,
             missing_types,
             created,
         )
@@ -565,12 +602,11 @@ class ServiceInfo(RecordUpdateListener):
         self, override_ttl: Optional[int] = None, created: Optional[float] = None
     ) -> Set[DNSRecord]:
         """Build a set of address records and NSEC records for non-present record types."""
-        seen_types: Set[int] = set()
+        missing_types: Set[int] = _ADDRESS_RECORD_TYPES.copy()
         records: Set[DNSRecord] = set()
         for dns_address in self.dns_addresses(override_ttl, IPVersion.All, created):
-            seen_types.add(dns_address.type)
+            missing_types.discard(dns_address.type)
             records.add(dns_address)
-        missing_types: Set[int] = _ADDRESS_RECORD_TYPES - seen_types
         if missing_types:
             assert self.server is not None, "Service server must be set for NSEC record."
             records.add(self.dns_nsec(list(missing_types), override_ttl, created))
@@ -588,20 +624,21 @@ class ServiceInfo(RecordUpdateListener):
         This function is for backwards compatibility.
         """
         if self.server is None:
-            self.server = self.name
+            self.server = self._name
             self.server_key = self.server.lower()
 
-    def load_from_cache(self, zc: 'Zeroconf') -> bool:
+    def load_from_cache(self, zc: 'Zeroconf', now: Optional[float] = None) -> bool:
         """Populate the service info from the cache.
 
         This method is designed to be threadsafe.
         """
-        now = current_time_millis()
+        if not now:
+            now = current_time_millis()
         original_server_key = self.server_key
-        cached_srv_record = zc.cache.get_by_details(self.name, _TYPE_SRV, _CLASS_IN)
+        cached_srv_record = zc.cache.get_by_details(self._name, _TYPE_SRV, _CLASS_IN)
         if cached_srv_record:
             self._process_record_threadsafe(zc, cached_srv_record, now)
-        cached_txt_record = zc.cache.get_by_details(self.name, _TYPE_TXT, _CLASS_IN)
+        cached_txt_record = zc.cache.get_by_details(self._name, _TYPE_TXT, _CLASS_IN)
         if cached_txt_record:
             self._process_record_threadsafe(zc, cached_txt_record, now)
         if original_server_key == self.server_key:
@@ -664,11 +701,13 @@ class ServiceInfo(RecordUpdateListener):
         """
         if not zc.started:
             await zc.async_wait_for_start()
-        if self.load_from_cache(zc):
+
+        now = current_time_millis()
+
+        if self.load_from_cache(zc, now):
             return True
 
         first_request = True
-        now = current_time_millis()
         delay = _LISTENER_TIME
         next_ = now
         last = now + timeout
@@ -683,7 +722,7 @@ class ServiceInfo(RecordUpdateListener):
                     )
                     first_request = False
                     if not out.questions:
-                        return self.load_from_cache(zc)
+                        return self.load_from_cache(zc, now)
                     zc.async_send(out, addr, port)
                     next_ = now + delay
                     delay *= 2
@@ -701,10 +740,13 @@ class ServiceInfo(RecordUpdateListener):
     ) -> DNSOutgoing:
         """Generate the request query."""
         out = DNSOutgoing(_FLAGS_QR_QUERY)
-        out.add_question_or_one_cache(zc.cache, now, self.name, _TYPE_SRV, _CLASS_IN)
-        out.add_question_or_one_cache(zc.cache, now, self.name, _TYPE_TXT, _CLASS_IN)
-        out.add_question_or_all_cache(zc.cache, now, self.server or self.name, _TYPE_A, _CLASS_IN)
-        out.add_question_or_all_cache(zc.cache, now, self.server or self.name, _TYPE_AAAA, _CLASS_IN)
+        name = self._name
+        server_or_name = self.server or name
+        cache = zc.cache
+        out.add_question_or_one_cache(cache, now, name, _TYPE_SRV, _CLASS_IN)
+        out.add_question_or_one_cache(cache, now, name, _TYPE_TXT, _CLASS_IN)
+        out.add_question_or_all_cache(cache, now, server_or_name, _TYPE_A, _CLASS_IN)
+        out.add_question_or_all_cache(cache, now, server_or_name, _TYPE_AAAA, _CLASS_IN)
         if question_type == DNSQuestionType.QU:
             for question in out.questions:
                 question.unicast = True
@@ -712,7 +754,7 @@ class ServiceInfo(RecordUpdateListener):
 
     def __eq__(self, other: object) -> bool:
         """Tests equality of service name"""
-        return isinstance(other, ServiceInfo) and other.name == self.name
+        return isinstance(other, ServiceInfo) and other._name == self._name
 
     def __repr__(self) -> str:
         """String representation"""
